"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class SentReceiptTypedDict(TypedDict):
    receipt_id: str
    r"""Unique identifier for the receipt."""
    idempotency_key: str
    r"""The idempotency key used when creating the receipt."""
    sent_on: datetime
    r"""When the receipt was sent."""


class SentReceipt(BaseModel):
    receipt_id: Annotated[str, pydantic.Field(alias="receiptID")]
    r"""Unique identifier for the receipt."""

    idempotency_key: Annotated[str, pydantic.Field(alias="idempotencyKey")]
    r"""The idempotency key used when creating the receipt."""

    sent_on: Annotated[datetime, pydantic.Field(alias="sentOn")]
    r"""When the receipt was sent."""


try:
    SentReceipt.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .sweepstatus import SweepStatus
from .sweepsubtotal import SweepSubtotal, SweepSubtotalTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SweepTypedDict(TypedDict):
    sweep_id: str
    status: SweepStatus
    accrued_amount: str
    r"""The total net amount of wallet transactions accrued in the sweep."""
    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    accrual_started_on: datetime
    r"""The date-time the sweep began accruing transactions."""
    residual_balance: NotRequired[str]
    r"""Amount remaining in the wallet after the sweep transfer completes."""
    accrual_ended_on: NotRequired[datetime]
    r"""The date-time the sweep stopped accruing transactions."""
    push_payment_method_id: NotRequired[str]
    r"""ID of the payment method."""
    pull_payment_method_id: NotRequired[str]
    r"""ID of the payment method."""
    transfer_id: NotRequired[str]
    transfer_amount: NotRequired[str]
    r"""Amount that is transferred."""
    statement_descriptor: NotRequired[str]
    r"""The text that appears on the banking statement. The default descriptor is a 10 character ID if an override is not set in the sweep configs statementDescriptor."""
    subtotals: NotRequired[List[SweepSubtotalTypedDict]]
    r"""The subtotal of each transaction type contributing to the sweep's accrued amount."""


class Sweep(BaseModel):
    sweep_id: Annotated[str, pydantic.Field(alias="sweepID")]

    status: SweepStatus

    accrued_amount: Annotated[str, pydantic.Field(alias="accruedAmount")]
    r"""The total net amount of wallet transactions accrued in the sweep."""

    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    accrual_started_on: Annotated[datetime, pydantic.Field(alias="accrualStartedOn")]
    r"""The date-time the sweep began accruing transactions."""

    residual_balance: Annotated[
        Optional[str], pydantic.Field(alias="residualBalance")
    ] = None
    r"""Amount remaining in the wallet after the sweep transfer completes."""

    accrual_ended_on: Annotated[
        Optional[datetime], pydantic.Field(alias="accrualEndedOn")
    ] = None
    r"""The date-time the sweep stopped accruing transactions."""

    push_payment_method_id: Annotated[
        Optional[str], pydantic.Field(alias="pushPaymentMethodID")
    ] = None
    r"""ID of the payment method."""

    pull_payment_method_id: Annotated[
        Optional[str], pydantic.Field(alias="pullPaymentMethodID")
    ] = None
    r"""ID of the payment method."""

    transfer_id: Annotated[Optional[str], pydantic.Field(alias="transferID")] = None

    transfer_amount: Annotated[
        Optional[str], pydantic.Field(alias="transferAmount")
    ] = None
    r"""Amount that is transferred."""

    statement_descriptor: Annotated[
        Optional[str], pydantic.Field(alias="statementDescriptor")
    ] = None
    r"""The text that appears on the banking statement. The default descriptor is a 10 character ID if an override is not set in the sweep configs statementDescriptor."""

    subtotals: Optional[List[SweepSubtotal]] = None
    r"""The subtotal of each transaction type contributing to the sweep's accrued amount."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "residualBalance",
                "accrualEndedOn",
                "pushPaymentMethodID",
                "pullPaymentMethodID",
                "transferID",
                "transferAmount",
                "statementDescriptor",
                "subtotals",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Sweep.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .reversedwithcancellation import (
    ReversedWithCancellation,
    ReversedWithCancellationTypedDict,
)
from .reversedwithrefund import ReversedWithRefund, ReversedWithRefundTypedDict
from typing import Union
from typing_extensions import TypeAliasType


ReversalTypedDict = TypeAliasType(
    "ReversalTypedDict",
    Union[ReversedWithCancellationTypedDict, ReversedWithRefundTypedDict],
)
r"""Contains either a cancellation or refund, depending on the method used to reverse the transfer."""


Reversal = TypeAliasType(
    "Reversal", Union[ReversedWithCancellation, ReversedWithRefund]
)
r"""Contains either a cancellation or refund, depending on the method used to reverse the transfer."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .issuedcardauthorizationeventresult import IssuedCardAuthorizationEventResult
from .issuedcardeventtype import IssuedCardEventType
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class IssuedCardAuthorizationEventTypedDict(TypedDict):
    event_id: str
    r"""The identifier for this event. Use the `eventType` field to determine what resource is identified by this ID (`authorization`, `reversal`, etc.)."""
    event_type: IssuedCardEventType
    r"""The type of event that occurred on the card."""
    amount: str
    r"""A decimal-formatted numerical string that represents up to 2 decimal place precision. In USD for example, 12.34 is $12.34 and 0.99 is $0.99."""
    result: IssuedCardAuthorizationEventResult
    r"""The result of an event."""
    created_on: datetime


class IssuedCardAuthorizationEvent(BaseModel):
    event_id: Annotated[str, pydantic.Field(alias="eventID")]
    r"""The identifier for this event. Use the `eventType` field to determine what resource is identified by this ID (`authorization`, `reversal`, etc.)."""

    event_type: Annotated[IssuedCardEventType, pydantic.Field(alias="eventType")]
    r"""The type of event that occurred on the card."""

    amount: str
    r"""A decimal-formatted numerical string that represents up to 2 decimal place precision. In USD for example, 12.34 is $12.34 and 0.99 is $0.99."""

    result: IssuedCardAuthorizationEventResult
    r"""The result of an event."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]


try:
    IssuedCardAuthorizationEvent.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .evidencetype import EvidenceType
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class DisputeEvidenceResponseTypedDict(TypedDict):
    evidence_id: str
    dispute_id: str
    evidence_type: EvidenceType
    created_on: datetime
    updated_on: datetime
    text: NotRequired[str]
    r"""For text evidence, the text submitted as evidence."""
    mime_type: NotRequired[str]
    r"""For file evidence, the MIME type of the file."""
    filename: NotRequired[str]
    r"""For file evidence, the name of the file."""
    size: NotRequired[int]
    r"""For file evidence, the size of the file."""
    submitted_on: NotRequired[datetime]
    r"""When the evidence was submitted for review."""


class DisputeEvidenceResponse(BaseModel):
    evidence_id: Annotated[str, pydantic.Field(alias="evidenceID")]

    dispute_id: Annotated[str, pydantic.Field(alias="disputeID")]

    evidence_type: Annotated[EvidenceType, pydantic.Field(alias="evidenceType")]

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    updated_on: Annotated[datetime, pydantic.Field(alias="updatedOn")]

    text: Optional[str] = None
    r"""For text evidence, the text submitted as evidence."""

    mime_type: Annotated[Optional[str], pydantic.Field(alias="mimeType")] = None
    r"""For file evidence, the MIME type of the file."""

    filename: Optional[str] = None
    r"""For file evidence, the name of the file."""

    size: Optional[int] = None
    r"""For file evidence, the size of the file."""

    submitted_on: Annotated[Optional[datetime], pydantic.Field(alias="submittedOn")] = (
        None
    )
    r"""When the evidence was submitted for review."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["text", "mimeType", "filename", "size", "submittedOn"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    DisputeEvidenceResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class ItemListNextPageTypedDict(TypedDict):
    cursor: str
    r"""The cursor value to use for the next page of items."""


class ItemListNextPage(BaseModel):
    cursor: str
    r"""The cursor value to use for the next page of items."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class MXAuthorizationCodeTypedDict(TypedDict):
    r"""The authorization code of a MX account which allows a processor to retrieve a linked payment account.

    `sandbox` - When linking a bank account to a `sandbox` account using a MX authorization code it will utilize MX's sandbox environment.
    The MX authorization code provided must be generated from MX's sandbox environment.
    """

    authorization_code: str


class MXAuthorizationCode(BaseModel):
    r"""The authorization code of a MX account which allows a processor to retrieve a linked payment account.

    `sandbox` - When linking a bank account to a `sandbox` account using a MX authorization code it will utilize MX's sandbox environment.
    The MX authorization code provided must be generated from MX's sandbox environment.
    """

    authorization_code: Annotated[str, pydantic.Field(alias="authorizationCode")]


try:
    MXAuthorizationCode.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .achpaymentdetails import ACHPaymentDetails, ACHPaymentDetailsTypedDict
from .cardpaymentdetails import CardPaymentDetails, CardPaymentDetailsTypedDict
from .collectionpaymentmethodtype import CollectionPaymentMethodType
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentLinkPaymentDetailsUpdateTypedDict(TypedDict):
    r"""Options for payment links used to collect payment."""

    allowed_methods: NotRequired[List[CollectionPaymentMethodType]]
    r"""A list of payment methods that should be supported for this payment link."""
    card_details: NotRequired[CardPaymentDetailsTypedDict]
    r"""Options for payment links used to collect a card payment."""
    ach_details: NotRequired[ACHPaymentDetailsTypedDict]
    r"""Options for payment links used to collect an ACH payment."""
    metadata: NotRequired[Dict[str, str]]
    r"""Optional free-form metadata for the transfer."""


class PaymentLinkPaymentDetailsUpdate(BaseModel):
    r"""Options for payment links used to collect payment."""

    allowed_methods: Annotated[
        Optional[List[CollectionPaymentMethodType]],
        pydantic.Field(alias="allowedMethods"),
    ] = None
    r"""A list of payment methods that should be supported for this payment link."""

    card_details: Annotated[
        Optional[CardPaymentDetails], pydantic.Field(alias="cardDetails")
    ] = None
    r"""Options for payment links used to collect a card payment."""

    ach_details: Annotated[
        Optional[ACHPaymentDetails], pydantic.Field(alias="achDetails")
    ] = None
    r"""Options for payment links used to collect an ACH payment."""

    metadata: Optional[Dict[str, str]] = None
    r"""Optional free-form metadata for the transfer."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["allowedMethods", "cardDetails", "achDetails", "metadata"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PaymentLinkPaymentDetailsUpdate.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .wallettransactionsourcetype import WalletTransactionSourceType
from .wallettransactionstatus import WalletTransactionStatus
from .wallettransactiontype import WalletTransactionType
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WalletTransactionTypedDict(TypedDict):
    r"""A transaction that funds or deducts from the wallet."""

    wallet_id: str
    transaction_id: str
    transaction_type: WalletTransactionType
    source_type: WalletTransactionSourceType
    source_id: str
    r"""The ID of the Moov object to which this transaction is related."""
    status: WalletTransactionStatus
    memo: str
    r"""Detailed description of the transaction."""
    created_on: datetime
    currency: str
    r"""A 3-letter ISO 4217 currency code."""
    gross_amount: int
    r"""The total transaction amount. The amount is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""
    gross_amount_decimal: str
    r"""The total transaction amount. Same as `grossAmount`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""
    fee: int
    r"""Total fees paid for the transaction. The value is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""
    fee_decimal: str
    r"""Total fees paid for the transaction. Same as `fee`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""
    net_amount: int
    r"""Net amount is the gross amount less fees paid, and the amount that affects the wallet's balance. The amount is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""
    net_amount_decimal: str
    r"""Net amount is the gross amount less fees paid, and the amount that affects the wallet's balance. Same as `netAmount`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""
    completed_on: NotRequired[datetime]
    fee_i_ds: NotRequired[List[str]]
    r"""The IDs of the fees paid for the transaction."""
    available_balance: NotRequired[int]
    r"""The wallet's total available balance after recording a completed transaction. The value is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""
    available_balance_decimal: NotRequired[str]
    r"""The wallet's total available balance after recording a completed transaction. Same as `availableBalance`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""
    sweep_id: NotRequired[str]
    r"""ID of the sweep this transaction accrued in."""


class WalletTransaction(BaseModel):
    r"""A transaction that funds or deducts from the wallet."""

    wallet_id: Annotated[str, pydantic.Field(alias="walletID")]

    transaction_id: Annotated[str, pydantic.Field(alias="transactionID")]

    transaction_type: Annotated[
        WalletTransactionType, pydantic.Field(alias="transactionType")
    ]

    source_type: Annotated[
        WalletTransactionSourceType, pydantic.Field(alias="sourceType")
    ]

    source_id: Annotated[str, pydantic.Field(alias="sourceID")]
    r"""The ID of the Moov object to which this transaction is related."""

    status: WalletTransactionStatus

    memo: str
    r"""Detailed description of the transaction."""

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    currency: str
    r"""A 3-letter ISO 4217 currency code."""

    gross_amount: Annotated[int, pydantic.Field(alias="grossAmount")]
    r"""The total transaction amount. The amount is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""

    gross_amount_decimal: Annotated[str, pydantic.Field(alias="grossAmountDecimal")]
    r"""The total transaction amount. Same as `grossAmount`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""

    fee: int
    r"""Total fees paid for the transaction. The value is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""

    fee_decimal: Annotated[str, pydantic.Field(alias="feeDecimal")]
    r"""Total fees paid for the transaction. Same as `fee`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""

    net_amount: Annotated[int, pydantic.Field(alias="netAmount")]
    r"""Net amount is the gross amount less fees paid, and the amount that affects the wallet's balance. The amount is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""

    net_amount_decimal: Annotated[str, pydantic.Field(alias="netAmountDecimal")]
    r"""Net amount is the gross amount less fees paid, and the amount that affects the wallet's balance. Same as `netAmount`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""

    completed_on: Annotated[Optional[datetime], pydantic.Field(alias="completedOn")] = (
        None
    )

    fee_i_ds: Annotated[Optional[List[str]], pydantic.Field(alias="feeIDs")] = None
    r"""The IDs of the fees paid for the transaction."""

    available_balance: Annotated[
        Optional[int], pydantic.Field(alias="availableBalance")
    ] = None
    r"""The wallet's total available balance after recording a completed transaction. The value is in the smallest unit of the specified currency. In USD this is cents, for example, $12.04 is 1204 and $0.99 is 99."""

    available_balance_decimal: Annotated[
        Optional[str], pydantic.Field(alias="availableBalanceDecimal")
    ] = None
    r"""The wallet's total available balance after recording a completed transaction. Same as `availableBalance`, but a decimal-formatted numerical string that represents up to 9 decimal place precision. In USD for example, 12.987654321 is $12.987654321 and 0.9987634521 is $0.9987634521."""

    sweep_id: Annotated[Optional[str], pydantic.Field(alias="sweepID")] = None
    r"""ID of the sweep this transaction accrued in."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "completedOn",
                "feeIDs",
                "availableBalance",
                "availableBalanceDecimal",
                "sweepID",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    WalletTransaction.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transactionsource import TransactionSource
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateTransferSourceCardTypedDict(TypedDict):
    dynamic_descriptor: NotRequired[str]
    r"""An optional override of the default card statement descriptor for a transfer. Accounts must be enabled by Moov to set this field."""
    transaction_source: NotRequired[TransactionSource]
    r"""Specifies the nature and initiator of a transaction.

    Crucial for recurring and merchant-initiated transactions as per card scheme rules.
    Omit for customer-initiated e-commerce transactions.
    """


class CreateTransferSourceCard(BaseModel):
    dynamic_descriptor: Annotated[
        Optional[str], pydantic.Field(alias="dynamicDescriptor")
    ] = None
    r"""An optional override of the default card statement descriptor for a transfer. Accounts must be enabled by Moov to set this field."""

    transaction_source: Annotated[
        Optional[TransactionSource], pydantic.Field(alias="transactionSource")
    ] = None
    r"""Specifies the nature and initiator of a transaction.

    Crucial for recurring and merchant-initiated transactions as per card scheme rules.
    Omit for customer-initiated e-commerce transactions.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["dynamicDescriptor", "transactionSource"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateTransferSourceCard.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .achlocation import AchLocation, AchLocationTypedDict
from moovio_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, TypedDict


class LogoTypedDict(TypedDict):
    name: str
    url: str


class Logo(BaseModel):
    name: str

    url: str


class AchParticipantTypedDict(TypedDict):
    ach_location: AchLocationTypedDict
    customer_name: str
    new_routing_number: str
    office_code: str
    phone_number: str
    record_type_code: str
    revised: str
    routing_number: str
    servicing_frb_number: str
    status_code: str
    view_code: str
    clean_name: str
    logo: Nullable[LogoTypedDict]


class AchParticipant(BaseModel):
    ach_location: Annotated[AchLocation, pydantic.Field(alias="achLocation")]

    customer_name: Annotated[str, pydantic.Field(alias="customerName")]

    new_routing_number: Annotated[str, pydantic.Field(alias="newRoutingNumber")]

    office_code: Annotated[str, pydantic.Field(alias="officeCode")]

    phone_number: Annotated[str, pydantic.Field(alias="phoneNumber")]

    record_type_code: Annotated[str, pydantic.Field(alias="recordTypeCode")]

    revised: str

    routing_number: Annotated[str, pydantic.Field(alias="routingNumber")]

    servicing_frb_number: Annotated[str, pydantic.Field(alias="servicingFRBNumber")]

    status_code: Annotated[str, pydantic.Field(alias="statusCode")]

    view_code: Annotated[str, pydantic.Field(alias="viewCode")]

    clean_name: Annotated[str, pydantic.Field(alias="cleanName")]

    logo: Nullable[Logo]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                m[k] = val

        return m


try:
    AchParticipant.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentmethodsbankaccount import (
    PaymentMethodsBankAccount,
    PaymentMethodsBankAccountTypedDict,
)
from enum import Enum
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class RtpCreditPaymentMethodPaymentMethodType(str, Enum):
    RTP_CREDIT = "rtp-credit"


class RtpCreditPaymentMethodTypedDict(TypedDict):
    payment_method_id: str
    r"""ID of the payment method."""
    payment_method_type: RtpCreditPaymentMethodPaymentMethodType
    bank_account: PaymentMethodsBankAccountTypedDict
    r"""A bank account as contained within a payment method."""


class RtpCreditPaymentMethod(BaseModel):
    payment_method_id: Annotated[str, pydantic.Field(alias="paymentMethodID")]
    r"""ID of the payment method."""

    payment_method_type: Annotated[
        RtpCreditPaymentMethodPaymentMethodType,
        pydantic.Field(alias="paymentMethodType"),
    ]

    bank_account: Annotated[
        PaymentMethodsBankAccount, pydantic.Field(alias="bankAccount")
    ]
    r"""A bank account as contained within a payment method."""


try:
    RtpCreditPaymentMethod.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class BillingSummaryInterchangeTypedDict(TypedDict):
    r"""A summary of interchange fees by card brand."""

    visa: AmountDecimalTypedDict
    r"""Total interchange fees for Visa."""
    mastercard: AmountDecimalTypedDict
    r"""Total interchange fees for Mastercard."""
    discover: AmountDecimalTypedDict
    r"""Total interchange fees for Discover."""
    american_express: AmountDecimalTypedDict
    r"""Total interchange fees for American Express."""


class BillingSummaryInterchange(BaseModel):
    r"""A summary of interchange fees by card brand."""

    visa: AmountDecimal
    r"""Total interchange fees for Visa."""

    mastercard: AmountDecimal
    r"""Total interchange fees for Mastercard."""

    discover: AmountDecimal
    r"""Total interchange fees for Discover."""

    american_express: Annotated[AmountDecimal, pydantic.Field(alias="americanExpress")]
    r"""Total interchange fees for American Express."""


try:
    BillingSummaryInterchange.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createruntransfer import CreateRunTransfer, CreateRunTransferTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class OccurrenceTypedDict(TypedDict):
    r"""Occurrences to either create or modify."""

    canceled: NotRequired[bool]
    r"""If set to true, will cancel the occurrence. If set false will resume the occurrence. If unset leaves the value unchanged."""
    occurrence_id: NotRequired[str]
    r"""If set this defines what occurrence to modify, if invalid will fail the request.
    If null or \"\" it defines to add a new occurrence.
    """
    run_on: NotRequired[datetime]
    r"""Timestamp to run the transfer after. Value must be into the future."""
    run_transfer: NotRequired[CreateRunTransferTypedDict]
    r"""Defines the attributes of a transfer."""


class Occurrence(BaseModel):
    r"""Occurrences to either create or modify."""

    canceled: Optional[bool] = None
    r"""If set to true, will cancel the occurrence. If set false will resume the occurrence. If unset leaves the value unchanged."""

    occurrence_id: Annotated[Optional[str], pydantic.Field(alias="occurrenceID")] = None
    r"""If set this defines what occurrence to modify, if invalid will fail the request.
    If null or \"\" it defines to add a new occurrence.
    """

    run_on: Annotated[Optional[datetime], pydantic.Field(alias="runOn")] = None
    r"""Timestamp to run the transfer after. Value must be into the future."""

    run_transfer: Annotated[
        Optional[CreateRunTransfer], pydantic.Field(alias="runTransfer")
    ] = None
    r"""Defines the attributes of a transfer."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["canceled", "occurrenceID", "runOn", "runTransfer"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Occurrence.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WalletType(str, Enum):
    r"""Type of a wallet.
    - `default`: The system-generated wallet automatically created when an account is granted the wallet capability.
    - `general`: An additional, user-defined wallet created via API or Dashboard.
    """

    DEFAULT = "default"
    GENERAL = "general"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import List, Mapping, Optional


class IssuingTransactions(BaseSDK):
    def list_authorizations(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        issued_card_id: Optional[str] = None,
        start_date_time: Optional[datetime] = None,
        end_date_time: Optional[datetime] = None,
        statuses: Optional[List[components.IssuingAuthorizationStatus]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListIssuedCardAuthorizationsResponse:
        r"""List issued card authorizations associated with a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param skip:
        :param count:
        :param issued_card_id: Optional ID of the issued card to filter results.
        :param start_date_time: Optional date-time which inclusively filters all authorizations created after this date-time.
        :param end_date_time: Optional date-time which exclusively filters all authorizations created before this date-time.
        :param statuses: Optional, comma-separated statuses of the authorization to filter results.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListIssuedCardAuthorizationsRequest(
            account_id=account_id,
            skip=skip,
            count=count,
            issued_card_id=issued_card_id,
            start_date_time=start_date_time,
            end_date_time=end_date_time,
            statuses=statuses,
        )

        req = self._build_request(
            method="GET",
            path="/issuing/{accountID}/authorizations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListIssuedCardAuthorizationsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIssuedCardAuthorizations",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListIssuedCardAuthorizationsResponse(
                result=unmarshal_json_response(
                    List[components.IssuedCardAuthorization], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_authorizations_async(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        issued_card_id: Optional[str] = None,
        start_date_time: Optional[datetime] = None,
        end_date_time: Optional[datetime] = None,
        statuses: Optional[List[components.IssuingAuthorizationStatus]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListIssuedCardAuthorizationsResponse:
        r"""List issued card authorizations associated with a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param skip:
        :param count:
        :param issued_card_id: Optional ID of the issued card to filter results.
        :param start_date_time: Optional date-time which inclusively filters all authorizations created after this date-time.
        :param end_date_time: Optional date-time which exclusively filters all authorizations created before this date-time.
        :param statuses: Optional, comma-separated statuses of the authorization to filter results.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListIssuedCardAuthorizationsRequest(
            account_id=account_id,
            skip=skip,
            count=count,
            issued_card_id=issued_card_id,
            start_date_time=start_date_time,
            end_date_time=end_date_time,
            statuses=statuses,
        )

        req = self._build_request_async(
            method="GET",
            path="/issuing/{accountID}/authorizations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListIssuedCardAuthorizationsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIssuedCardAuthorizations",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListIssuedCardAuthorizationsResponse(
                result=unmarshal_json_response(
                    List[components.IssuedCardAuthorization], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_authorization(
        self,
        *,
        account_id: str,
        authorization_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetIssuedCardAuthorizationResponse:
        r"""Retrieves details of an authorization associated with a specific Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param authorization_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetIssuedCardAuthorizationRequest(
            account_id=account_id,
            authorization_id=authorization_id,
        )

        req = self._build_request(
            method="GET",
            path="/issuing/{accountID}/authorizations/{authorizationID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetIssuedCardAuthorizationGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIssuedCardAuthorization",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetIssuedCardAuthorizationResponse(
                result=unmarshal_json_response(
                    components.IssuedCardAuthorization, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_authorization_async(
        self,
        *,
        account_id: str,
        authorization_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetIssuedCardAuthorizationResponse:
        r"""Retrieves details of an authorization associated with a specific Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param authorization_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetIssuedCardAuthorizationRequest(
            account_id=account_id,
            authorization_id=authorization_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/issuing/{accountID}/authorizations/{authorizationID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetIssuedCardAuthorizationGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIssuedCardAuthorization",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetIssuedCardAuthorizationResponse(
                result=unmarshal_json_response(
                    components.IssuedCardAuthorization, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list_authorization_events(
        self,
        *,
        account_id: str,
        authorization_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListIssuedCardAuthorizationEventsResponse:
        r"""List card network and Moov platform events that affect the authorization and its hold on a wallet balance.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param authorization_id:
        :param skip:
        :param count:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListIssuedCardAuthorizationEventsRequest(
            account_id=account_id,
            authorization_id=authorization_id,
            skip=skip,
            count=count,
        )

        req = self._build_request(
            method="GET",
            path="/issuing/{accountID}/authorizations/{authorizationID}/events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListIssuedCardAuthorizationEventsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIssuedCardAuthorizationEvents",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListIssuedCardAuthorizationEventsResponse(
                result=unmarshal_json_response(
                    List[components.IssuedCardAuthorizationEvent], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_authorization_events_async(
        self,
        *,
        account_id: str,
        authorization_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListIssuedCardAuthorizationEventsResponse:
        r"""List card network and Moov platform events that affect the authorization and its hold on a wallet balance.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param authorization_id:
        :param skip:
        :param count:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListIssuedCardAuthorizationEventsRequest(
            account_id=account_id,
            authorization_id=authorization_id,
            skip=skip,
            count=count,
        )

        req = self._build_request_async(
            method="GET",
            path="/issuing/{accountID}/authorizations/{authorizationID}/events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListIssuedCardAuthorizationEventsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIssuedCardAuthorizationEvents",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListIssuedCardAuthorizationEventsResponse(
                result=unmarshal_json_response(
                    List[components.IssuedCardAuthorizationEvent], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def list(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        issued_card_id: Optional[str] = None,
        start_date_time: Optional[datetime] = None,
        end_date_time: Optional[datetime] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListIssuedCardTransactionsResponse:
        r"""List issued card transactions associated with a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param skip:
        :param count:
        :param issued_card_id: Optional ID of the issued card to filter results.
        :param start_date_time: Optional date-time which inclusively filters all card transactions created after this date-time.
        :param end_date_time: Optional date-time which exclusively filters all card transactions created before this date-time.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListIssuedCardTransactionsRequest(
            account_id=account_id,
            skip=skip,
            count=count,
            issued_card_id=issued_card_id,
            start_date_time=start_date_time,
            end_date_time=end_date_time,
        )

        req = self._build_request(
            method="GET",
            path="/issuing/{accountID}/card-transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListIssuedCardTransactionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIssuedCardTransactions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListIssuedCardTransactionsResponse(
                result=unmarshal_json_response(
                    List[components.IssuedCardTransaction], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        account_id: str,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        issued_card_id: Optional[str] = None,
        start_date_time: Optional[datetime] = None,
        end_date_time: Optional[datetime] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListIssuedCardTransactionsResponse:
        r"""List issued card transactions associated with a Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param skip:
        :param count:
        :param issued_card_id: Optional ID of the issued card to filter results.
        :param start_date_time: Optional date-time which inclusively filters all card transactions created after this date-time.
        :param end_date_time: Optional date-time which exclusively filters all card transactions created before this date-time.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListIssuedCardTransactionsRequest(
            account_id=account_id,
            skip=skip,
            count=count,
            issued_card_id=issued_card_id,
            start_date_time=start_date_time,
            end_date_time=end_date_time,
        )

        req = self._build_request_async(
            method="GET",
            path="/issuing/{accountID}/card-transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListIssuedCardTransactionsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listIssuedCardTransactions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListIssuedCardTransactionsResponse(
                result=unmarshal_json_response(
                    List[components.IssuedCardTransaction], http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        card_transaction_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetIssuedCardTransactionResponse:
        r"""Retrieves details of an issued card transaction associated with a specific Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param card_transaction_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetIssuedCardTransactionRequest(
            account_id=account_id,
            card_transaction_id=card_transaction_id,
        )

        req = self._build_request(
            method="GET",
            path="/issuing/{accountID}/card-transactions/{cardTransactionID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetIssuedCardTransactionGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIssuedCardTransaction",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetIssuedCardTransactionResponse(
                result=unmarshal_json_response(
                    components.IssuedCardTransaction, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        account_id: str,
        card_transaction_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetIssuedCardTransactionResponse:
        r"""Retrieves details of an issued card transaction associated with a specific Moov account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/issued-cards.read` scope.

        :param account_id: The Moov business account for which cards have been issued.
        :param card_transaction_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetIssuedCardTransactionRequest(
            account_id=account_id,
            card_transaction_id=card_transaction_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/issuing/{accountID}/card-transactions/{cardTransactionID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetIssuedCardTransactionGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getIssuedCardTransaction",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetIssuedCardTransactionResponse(
                result=unmarshal_json_response(
                    components.IssuedCardTransaction, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CardType(str, Enum):
    r"""The type of the card."""

    DEBIT = "debit"
    CREDIT = "credit"
    PREPAID = "prepaid"
    UNKNOWN = "unknown"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class Mode(str, Enum):
    r"""The operating mode for an account."""

    SANDBOX = "sandbox"
    PRODUCTION = "production"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
from typing_extensions import deprecated


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class UnderwritingStatus(str, Enum):
    APPROVED = "approved"
    REJECTED = "rejected"
    PENDING_REVIEW = "pendingReview"
    PENDING = "pending"
    NOT_REQUESTED = "notRequested"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import httpx
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class GetPublicImageRequestTypedDict(TypedDict):
    public_id: str
    if_none_match: NotRequired[str]
    size: NotRequired[str]
    r"""Optional parameter to request a resized version of the image (WxH).

    If either dimension is 0, the image will be scaled proportionally based on
    the non-zero dimension. Dimensions are capped at 2048 pixels. A default size
    of 400x400 will be used if this parameter is omitted.
    """


class GetPublicImageRequest(BaseModel):
    public_id: Annotated[
        str,
        pydantic.Field(alias="publicID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    if_none_match: Annotated[
        Optional[str],
        pydantic.Field(alias="if-none-match"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None

    size: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = "400x400"
    r"""Optional parameter to request a resized version of the image (WxH).

    If either dimension is 0, the image will be scaled proportionally based on
    the non-zero dimension. Dimensions are capped at 2048 pixels. A default size
    of 400x400 will be used if this parameter is omitted.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["if-none-match", "size"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


GetPublicImageResponseResultTypedDict = TypeAliasType(
    "GetPublicImageResponseResultTypedDict",
    Union[httpx.Response, httpx.Response, httpx.Response],
)


GetPublicImageResponseResult = TypeAliasType(
    "GetPublicImageResponseResult",
    Union[httpx.Response, httpx.Response, httpx.Response],
)


class GetPublicImageResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: GetPublicImageResponseResultTypedDict


class GetPublicImageResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: GetPublicImageResponseResult



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CreateRefundTypedDict(TypedDict):
    r"""Specifies a partial amount to refund.

    This request body is optional, an empty body will issue a refund for the full amount of the original transfer.
    """

    amount: NotRequired[int]
    r"""Amount to refund in cents. If null, the original transfer's full amount will be refunded."""


class CreateRefund(BaseModel):
    r"""Specifies a partial amount to refund.

    This request body is optional, an empty body will issue a refund for the full amount of the original transfer.
    """

    amount: Optional[int] = None
    r"""Amount to refund in cents. If null, the original transfer's full amount will be refunded."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["amount"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardtransactionfailurecode import CardTransactionFailureCode
from .refundcardstatus import RefundCardStatus
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class RefundCardDetailsTypedDict(TypedDict):
    status: RefundCardStatus
    failure_code: NotRequired[CardTransactionFailureCode]
    initiated_on: NotRequired[datetime]
    confirmed_on: NotRequired[datetime]
    settled_on: NotRequired[datetime]
    failed_on: NotRequired[datetime]
    completed_on: NotRequired[datetime]


class RefundCardDetails(BaseModel):
    status: RefundCardStatus

    failure_code: Annotated[
        Optional[CardTransactionFailureCode], pydantic.Field(alias="failureCode")
    ] = None

    initiated_on: Annotated[Optional[datetime], pydantic.Field(alias="initiatedOn")] = (
        None
    )

    confirmed_on: Annotated[Optional[datetime], pydantic.Field(alias="confirmedOn")] = (
        None
    )

    settled_on: Annotated[Optional[datetime], pydantic.Field(alias="settledOn")] = None

    failed_on: Annotated[Optional[datetime], pydantic.Field(alias="failedOn")] = None

    completed_on: Annotated[Optional[datetime], pydantic.Field(alias="completedOn")] = (
        None
    )

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "failureCode",
                "initiatedOn",
                "confirmedOn",
                "settledOn",
                "failedOn",
                "completedOn",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    RefundCardDetails.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class GrantType(str, Enum):
    r"""The type of grant being requested.

    - `client_credentials`: A grant type used by clients to obtain an access token
    - `refresh_token`: A grant type used by clients to obtain a new access token using a refresh token
    """

    CLIENT_CREDENTIALS = "client_credentials"
    REFRESH_TOKEN = "refresh_token"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .businessprofile import BusinessProfile, BusinessProfileTypedDict
from .guestprofile import GuestProfile, GuestProfileTypedDict
from .individualprofile import IndividualProfile, IndividualProfileTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ProfileTypedDict(TypedDict):
    r"""Describes a Moov account profile. A profile will have a business, individual, or guest depending on the account's type."""

    individual: NotRequired[IndividualProfileTypedDict]
    r"""Describes an individual."""
    business: NotRequired[BusinessProfileTypedDict]
    r"""Describes a business."""
    guest: NotRequired[GuestProfileTypedDict]
    r"""Describes a guest account profile."""


class Profile(BaseModel):
    r"""Describes a Moov account profile. A profile will have a business, individual, or guest depending on the account's type."""

    individual: Optional[IndividualProfile] = None
    r"""Describes an individual."""

    business: Optional[BusinessProfile] = None
    r"""Describes a business."""

    guest: Optional[GuestProfile] = None
    r"""Describes a guest account profile."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["individual", "business", "guest"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .generatedby import GeneratedBy, GeneratedByTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IncurredFeeTypedDict(TypedDict):
    r"""A fee incurred by a user."""

    fee_id: NotRequired[str]
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""
    account_id: NotRequired[str]
    r"""Account ID the fee belongs to."""
    wallet_id: NotRequired[str]
    r"""Wallet ID associated with the fee."""
    created_on: NotRequired[datetime]
    r"""Timestamp when the fee was created."""
    fee_name: NotRequired[str]
    r"""Name of the fee."""
    amount: NotRequired[AmountDecimalTypedDict]
    r"""The fee amount."""
    generated_by: NotRequired[GeneratedByTypedDict]
    r"""Object indicating what generated the fee (TransferID, CardID, DisputeID, AccountID, or BankAccountID)."""
    fee_group: NotRequired[str]
    r"""Describes the source of the fee, such as a Moov-set processing fee, a network pass-through fee, or an interchange or discount fee."""
    residual_id: NotRequired[str]
    r"""Unique identifier for this residual payment calculation."""


class IncurredFee(BaseModel):
    r"""A fee incurred by a user."""

    fee_id: Annotated[Optional[str], pydantic.Field(alias="feeID")] = None
    r"""A unique identifier for a Moov resource. Supports UUID format (xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx) or typed format with base32-encoded UUID and type suffix (e.g., kuoaydiojf7uszaokc2ggnaaaa_xfer)."""

    account_id: Annotated[Optional[str], pydantic.Field(alias="accountID")] = None
    r"""Account ID the fee belongs to."""

    wallet_id: Annotated[Optional[str], pydantic.Field(alias="walletID")] = None
    r"""Wallet ID associated with the fee."""

    created_on: Annotated[Optional[datetime], pydantic.Field(alias="createdOn")] = None
    r"""Timestamp when the fee was created."""

    fee_name: Annotated[Optional[str], pydantic.Field(alias="feeName")] = None
    r"""Name of the fee."""

    amount: Optional[AmountDecimal] = None
    r"""The fee amount."""

    generated_by: Annotated[
        Optional[GeneratedBy], pydantic.Field(alias="generatedBy")
    ] = None
    r"""Object indicating what generated the fee (TransferID, CardID, DisputeID, AccountID, or BankAccountID)."""

    fee_group: Annotated[Optional[str], pydantic.Field(alias="feeGroup")] = None
    r"""Describes the source of the fee, such as a Moov-set processing fee, a network pass-through fee, or an interchange or discount fee."""

    residual_id: Annotated[Optional[str], pydantic.Field(alias="residualID")] = None
    r"""Unique identifier for this residual payment calculation."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "feeID",
                "accountID",
                "walletID",
                "createdOn",
                "feeName",
                "amount",
                "generatedBy",
                "feeGroup",
                "residualID",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    IncurredFee.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardvolumedistribution import (
    CardVolumeDistribution,
    CardVolumeDistributionTypedDict,
)
from .fulfillmentdetails import FulfillmentDetails, FulfillmentDetailsTypedDict
from .volumebycustomertype import VolumeByCustomerType, VolumeByCustomerTypeTypedDict
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class UpdateUnderwritingTypedDict(TypedDict):
    average_transaction_size: int
    max_transaction_size: int
    average_monthly_transaction_volume: int
    volume_by_customer_type: VolumeByCustomerTypeTypedDict
    card_volume_distribution: CardVolumeDistributionTypedDict
    fulfillment: FulfillmentDetailsTypedDict


class UpdateUnderwriting(BaseModel):
    average_transaction_size: Annotated[
        int, pydantic.Field(alias="averageTransactionSize")
    ]

    max_transaction_size: Annotated[int, pydantic.Field(alias="maxTransactionSize")]

    average_monthly_transaction_volume: Annotated[
        int, pydantic.Field(alias="averageMonthlyTransactionVolume")
    ]

    volume_by_customer_type: Annotated[
        VolumeByCustomerType, pydantic.Field(alias="volumeByCustomerType")
    ]

    card_volume_distribution: Annotated[
        CardVolumeDistribution, pydantic.Field(alias="cardVolumeDistribution")
    ]

    fulfillment: FulfillmentDetails


try:
    UpdateUnderwriting.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardaccountupdater import CardAccountUpdater, CardAccountUpdaterTypedDict
from .cardaddress import CardAddress, CardAddressTypedDict
from .cardbrand import CardBrand
from .cardexpiration import CardExpiration, CardExpirationTypedDict
from .cardtype import CardType
from .cardverification import CardVerification, CardVerificationTypedDict
from .domesticpullfromcard import DomesticPullFromCard
from .domesticpushtocard import DomesticPushToCard
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TransferPaymentMethodsCardTypedDict(TypedDict):
    r"""A card as contained within a payment method."""

    card_id: str
    r"""ID of the card."""
    fingerprint: str
    r"""Uniquely identifies a linked payment card or token.
    For Apple Pay, the fingerprint is based on the tokenized card number and may vary based on the user's device.
    This field can be used to identify specific payment methods across multiple accounts on your platform.
    """
    brand: CardBrand
    r"""The card brand."""
    card_type: CardType
    r"""The type of the card."""
    last_four_card_number: str
    r"""Last four digits of the card number"""
    bin: str
    r"""The first six to eight digits of the card number, which identifies the financial institution that issued the card."""
    expiration: CardExpirationTypedDict
    r"""The expiration date of the card or token."""
    billing_address: CardAddressTypedDict
    card_verification: CardVerificationTypedDict
    r"""The results of submitting cardholder data to a card network for verification."""
    holder_name: NotRequired[str]
    r"""The name of the cardholder as it appears on the card."""
    issuer: NotRequired[str]
    r"""Financial institution that issued the card."""
    issuer_country: NotRequired[str]
    r"""Country where the card was issued."""
    card_on_file: NotRequired[bool]
    r"""Indicates cardholder has authorized card to be stored for future payments."""
    merchant_account_id: NotRequired[str]
    card_account_updater: NotRequired[CardAccountUpdaterTypedDict]
    r"""The results of the most recent card update request."""
    domestic_push_to_card: NotRequired[DomesticPushToCard]
    r"""Indicates which level of domestic push-to-card transfer is supported by the card, if any."""
    domestic_pull_from_card: NotRequired[DomesticPullFromCard]
    r"""Indicates if the card supports domestic pull-from-card transfer."""


class TransferPaymentMethodsCard(BaseModel):
    r"""A card as contained within a payment method."""

    card_id: Annotated[str, pydantic.Field(alias="cardID")]
    r"""ID of the card."""

    fingerprint: str
    r"""Uniquely identifies a linked payment card or token.
    For Apple Pay, the fingerprint is based on the tokenized card number and may vary based on the user's device.
    This field can be used to identify specific payment methods across multiple accounts on your platform.
    """

    brand: CardBrand
    r"""The card brand."""

    card_type: Annotated[CardType, pydantic.Field(alias="cardType")]
    r"""The type of the card."""

    last_four_card_number: Annotated[str, pydantic.Field(alias="lastFourCardNumber")]
    r"""Last four digits of the card number"""

    bin: str
    r"""The first six to eight digits of the card number, which identifies the financial institution that issued the card."""

    expiration: CardExpiration
    r"""The expiration date of the card or token."""

    billing_address: Annotated[CardAddress, pydantic.Field(alias="billingAddress")]

    card_verification: Annotated[
        CardVerification, pydantic.Field(alias="cardVerification")
    ]
    r"""The results of submitting cardholder data to a card network for verification."""

    holder_name: Annotated[Optional[str], pydantic.Field(alias="holderName")] = None
    r"""The name of the cardholder as it appears on the card."""

    issuer: Optional[str] = None
    r"""Financial institution that issued the card."""

    issuer_country: Annotated[Optional[str], pydantic.Field(alias="issuerCountry")] = (
        None
    )
    r"""Country where the card was issued."""

    card_on_file: Annotated[Optional[bool], pydantic.Field(alias="cardOnFile")] = None
    r"""Indicates cardholder has authorized card to be stored for future payments."""

    merchant_account_id: Annotated[
        Optional[str], pydantic.Field(alias="merchantAccountID")
    ] = None

    card_account_updater: Annotated[
        Optional[CardAccountUpdater], pydantic.Field(alias="cardAccountUpdater")
    ] = None
    r"""The results of the most recent card update request."""

    domestic_push_to_card: Annotated[
        Optional[DomesticPushToCard], pydantic.Field(alias="domesticPushToCard")
    ] = None
    r"""Indicates which level of domestic push-to-card transfer is supported by the card, if any."""

    domestic_pull_from_card: Annotated[
        Optional[DomesticPullFromCard], pydantic.Field(alias="domesticPullFromCard")
    ] = None
    r"""Indicates if the card supports domestic pull-from-card transfer."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "holderName",
                "issuer",
                "issuerCountry",
                "cardOnFile",
                "merchantAccountID",
                "cardAccountUpdater",
                "domesticPushToCard",
                "domesticPullFromCard",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TransferPaymentMethodsCard.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ApplePayHeaderTypedDict(TypedDict):
    r"""Information needed to decrypt Apple Pay payment data.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/passkit/payment-token-format-reference#Header-keys-and-values)
    for more information.
    """

    public_key_hash: str
    r"""A base64-encoded, SHA-256 hash of the merchant's public key."""
    transaction_id: str
    r"""A device-generated identifier for the transaction."""
    ephemeral_public_key: NotRequired[str]
    r"""Base64-encoded ephemeral public key, used for ECC-encrypted payment data."""


class ApplePayHeader(BaseModel):
    r"""Information needed to decrypt Apple Pay payment data.

    Refer to [Apple's documentation](https://developer.apple.com/documentation/passkit/payment-token-format-reference#Header-keys-and-values)
    for more information.
    """

    public_key_hash: Annotated[str, pydantic.Field(alias="publicKeyHash")]
    r"""A base64-encoded, SHA-256 hash of the merchant's public key."""

    transaction_id: Annotated[str, pydantic.Field(alias="transactionId")]
    r"""A device-generated identifier for the transaction."""

    ephemeral_public_key: Annotated[
        Optional[str], pydantic.Field(alias="ephemeralPublicKey")
    ] = None
    r"""Base64-encoded ephemeral public key, used for ECC-encrypted payment data."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["ephemeralPublicKey"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ApplePayHeader.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class BrandColorTypedDict(TypedDict):
    accent: str
    r"""A CSS hex color representing an accent color."""


class BrandColor(BaseModel):
    accent: str
    r"""A CSS hex color representing an accent color."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .accountnameverification import (
    AccountNameVerification,
    AccountNameVerificationTypedDict,
)
from .cardverificationresult import CardVerificationResult
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CardVerificationTypedDict(TypedDict):
    r"""The results of submitting cardholder data to a card network for verification."""

    cvv: CardVerificationResult
    address_line1: CardVerificationResult
    postal_code: CardVerificationResult
    account_name: NotRequired[AccountNameVerificationTypedDict]
    r"""The results of submitting cardholder name to a card network for verification."""


class CardVerification(BaseModel):
    r"""The results of submitting cardholder data to a card network for verification."""

    cvv: CardVerificationResult

    address_line1: Annotated[
        CardVerificationResult, pydantic.Field(alias="addressLine1")
    ]

    postal_code: Annotated[CardVerificationResult, pydantic.Field(alias="postalCode")]

    account_name: Annotated[
        Optional[AccountNameVerification], pydantic.Field(alias="accountName")
    ] = None
    r"""The results of submitting cardholder name to a card network for verification."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["accountName"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CardVerification.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookTransferPaymentMethodDetailsTypedDict(TypedDict):
    r"""Payment method details for the source or destination of a transfer."""

    account_id: str
    payment_method_id: str


class WebhookTransferPaymentMethodDetails(BaseModel):
    r"""Payment method details for the source or destination of a transfer."""

    account_id: Annotated[str, pydantic.Field(alias="accountID")]

    payment_method_id: Annotated[str, pydantic.Field(alias="paymentMethodID")]


try:
    WebhookTransferPaymentMethodDetails.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from .cancellation import Cancellation, CancellationTypedDict
from .cardacquiringdispute import CardAcquiringDispute, CardAcquiringDisputeTypedDict
from .cardacquiringrefund import CardAcquiringRefund, CardAcquiringRefundTypedDict
from .facilitatorfee import FacilitatorFee, FacilitatorFeeTypedDict
from .moovfeedetails import MoovFeeDetails, MoovFeeDetailsTypedDict
from .transferdestination import TransferDestination, TransferDestinationTypedDict
from .transferfailurereason import TransferFailureReason
from .transferlineitems import TransferLineItems, TransferLineItemsTypedDict
from .transfersource import TransferSource, TransferSourceTypedDict
from .transferstatus import TransferStatus
from datetime import datetime
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreatedTransferTypedDict(TypedDict):
    transfer_id: str
    created_on: datetime
    source: NotRequired[TransferSourceTypedDict]
    destination: NotRequired[TransferDestinationTypedDict]
    completed_on: NotRequired[datetime]
    status: NotRequired[TransferStatus]
    r"""Status of a transfer."""
    failure_reason: NotRequired[TransferFailureReason]
    r"""Reason for a transfer's failure."""
    amount: NotRequired[AmountTypedDict]
    description: NotRequired[str]
    r"""An optional description of the transfer that is used on receipts and for your own internal use."""
    metadata: NotRequired[Dict[str, str]]
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""
    facilitator_fee: NotRequired[FacilitatorFeeTypedDict]
    r"""Total or markup fee."""
    moov_fee: NotRequired[int]
    r"""Fees charged to your platform account for transfers."""
    moov_fee_decimal: NotRequired[str]
    r"""Same as `moovFee`, but a decimal-formatted numerical string that represents up to 9 decimal place precision."""
    moov_fee_details: NotRequired[MoovFeeDetailsTypedDict]
    r"""Processing and pass-through costs that add up to the moovFee."""
    group_id: NotRequired[str]
    cancellations: NotRequired[List[CancellationTypedDict]]
    refunded_amount: NotRequired[AmountTypedDict]
    refunds: NotRequired[List[CardAcquiringRefundTypedDict]]
    disputed_amount: NotRequired[AmountTypedDict]
    disputes: NotRequired[List[CardAcquiringDisputeTypedDict]]
    sweep_id: NotRequired[str]
    schedule_id: NotRequired[str]
    occurrence_id: NotRequired[str]
    payment_link_code: NotRequired[str]
    sales_tax_amount: NotRequired[AmountTypedDict]
    r"""Optional sales tax amount. `transfer.amount.value` should be inclusive of any sales tax and represents the total amount charged."""
    foreign_id: NotRequired[str]
    r"""Optional alias from a foreign/external system which can be used to reference this resource."""
    line_items: NotRequired[TransferLineItemsTypedDict]
    r"""An optional collection of line items for a transfer.
    When line items are provided, their total plus sales tax must equal the transfer amount.
    """


class CreatedTransfer(BaseModel):
    transfer_id: Annotated[str, pydantic.Field(alias="transferID")]

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    source: Optional[TransferSource] = None

    destination: Optional[TransferDestination] = None

    completed_on: Annotated[Optional[datetime], pydantic.Field(alias="completedOn")] = (
        None
    )

    status: Optional[TransferStatus] = None
    r"""Status of a transfer."""

    failure_reason: Annotated[
        Optional[TransferFailureReason], pydantic.Field(alias="failureReason")
    ] = None
    r"""Reason for a transfer's failure."""

    amount: Optional[Amount] = None

    description: Optional[str] = None
    r"""An optional description of the transfer that is used on receipts and for your own internal use."""

    metadata: Optional[Dict[str, str]] = None
    r"""Free-form key-value pair list. Useful for storing information that is not captured elsewhere."""

    facilitator_fee: Annotated[
        Optional[FacilitatorFee], pydantic.Field(alias="facilitatorFee")
    ] = None
    r"""Total or markup fee."""

    moov_fee: Annotated[Optional[int], pydantic.Field(alias="moovFee")] = None
    r"""Fees charged to your platform account for transfers."""

    moov_fee_decimal: Annotated[
        Optional[str], pydantic.Field(alias="moovFeeDecimal")
    ] = None
    r"""Same as `moovFee`, but a decimal-formatted numerical string that represents up to 9 decimal place precision."""

    moov_fee_details: Annotated[
        Optional[MoovFeeDetails], pydantic.Field(alias="moovFeeDetails")
    ] = None
    r"""Processing and pass-through costs that add up to the moovFee."""

    group_id: Annotated[Optional[str], pydantic.Field(alias="groupID")] = None

    cancellations: Optional[List[Cancellation]] = None

    refunded_amount: Annotated[
        Optional[Amount], pydantic.Field(alias="refundedAmount")
    ] = None

    refunds: Optional[List[CardAcquiringRefund]] = None

    disputed_amount: Annotated[
        Optional[Amount], pydantic.Field(alias="disputedAmount")
    ] = None

    disputes: Optional[List[CardAcquiringDispute]] = None

    sweep_id: Annotated[Optional[str], pydantic.Field(alias="sweepID")] = None

    schedule_id: Annotated[Optional[str], pydantic.Field(alias="scheduleID")] = None

    occurrence_id: Annotated[Optional[str], pydantic.Field(alias="occurrenceID")] = None

    payment_link_code: Annotated[
        Optional[str], pydantic.Field(alias="paymentLinkCode")
    ] = None

    sales_tax_amount: Annotated[
        Optional[Amount], pydantic.Field(alias="salesTaxAmount")
    ] = None
    r"""Optional sales tax amount. `transfer.amount.value` should be inclusive of any sales tax and represents the total amount charged."""

    foreign_id: Annotated[Optional[str], pydantic.Field(alias="foreignID")] = None
    r"""Optional alias from a foreign/external system which can be used to reference this resource."""

    line_items: Annotated[
        Optional[TransferLineItems], pydantic.Field(alias="lineItems")
    ] = None
    r"""An optional collection of line items for a transfer.
    When line items are provided, their total plus sales tax must equal the transfer amount.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "source",
                "destination",
                "completedOn",
                "status",
                "failureReason",
                "amount",
                "description",
                "metadata",
                "facilitatorFee",
                "moovFee",
                "moovFeeDecimal",
                "moovFeeDetails",
                "groupID",
                "cancellations",
                "refundedAmount",
                "refunds",
                "disputedAmount",
                "disputes",
                "sweepID",
                "scheduleID",
                "occurrenceID",
                "paymentLinkCode",
                "salesTaxAmount",
                "foreignID",
                "lineItems",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreatedTransfer.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.models.components import (
    createinvoicepayment as components_createinvoicepayment,
    invoicepayment as components_invoicepayment,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateInvoicePaymentGlobalsTypedDict(TypedDict):
    x_moov_version: NotRequired[str]
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """


class CreateInvoicePaymentGlobals(BaseModel):
    x_moov_version: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Moov-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["X-Moov-Version"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateInvoicePaymentRequestTypedDict(TypedDict):
    account_id: str
    invoice_id: str
    create_invoice_payment: (
        components_createinvoicepayment.CreateInvoicePaymentTypedDict
    )


class CreateInvoicePaymentRequest(BaseModel):
    account_id: Annotated[
        str,
        pydantic.Field(alias="accountID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    invoice_id: Annotated[
        str,
        pydantic.Field(alias="invoiceID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    create_invoice_payment: Annotated[
        components_createinvoicepayment.CreateInvoicePayment,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class CreateInvoicePaymentResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: components_invoicepayment.InvoicePaymentTypedDict


class CreateInvoicePaymentResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: components_invoicepayment.InvoicePayment



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .terminalapplicationstatus import TerminalApplicationStatus
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class WebhookDataTerminalApplicationCreatedTypedDict(TypedDict):
    terminal_application_id: str
    status: TerminalApplicationStatus
    r"""Status of the terminal application."""


class WebhookDataTerminalApplicationCreated(BaseModel):
    terminal_application_id: Annotated[
        str, pydantic.Field(alias="terminalApplicationID")
    ]

    status: TerminalApplicationStatus
    r"""Status of the terminal application."""


try:
    WebhookDataTerminalApplicationCreated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class MonthlyVolumeRange(str, Enum):
    r"""The low value in each range is included. The high value in each range is excluded."""

    UNDER_10K = "under-10k"
    TENK_50K = "10k-50k"
    FIFTYK_100K = "50k-100k"
    ONE_HUNDREDK_250K = "100k-250k"
    TWO_HUNDRED_AND_FIFTYK_500K = "250k-500k"
    FIVE_HUNDREDK_1M = "500k-1m"
    ONEM_5M = "1m-5m"
    OVER_5M = "over-5m"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CapabilityID(str, Enum):
    r"""Moov account capabilities.

    The `production-app`, `platform.production-app`, and / or `platform.wallet-transfers` capabilities might appear in your list. These are read-only capabilities that Moov requests and uses for account verification purposes. These capabilities remains active with your account and require no additional action.
    """

    TRANSFERS = "transfers"
    SEND_FUNDS = "send-funds"
    SEND_FUNDS_PUSH_TO_CARD = "send-funds.push-to-card"
    MONEY_TRANSFER_PUSH_TO_CARD = "money-transfer.push-to-card"
    SEND_FUNDS_ACH = "send-funds.ach"
    SEND_FUNDS_RTP = "send-funds.rtp"
    SEND_FUNDS_INSTANT_BANK = "send-funds.instant-bank"
    COLLECT_FUNDS = "collect-funds"
    COLLECT_FUNDS_CARD_PAYMENTS = "collect-funds.card-payments"
    MONEY_TRANSFER_PULL_FROM_CARD = "money-transfer.pull-from-card"
    COLLECT_FUNDS_ACH = "collect-funds.ach"
    WALLET = "wallet"
    WALLET_BALANCE = "wallet.balance"
    CARD_ISSUING = "card-issuing"
    PRODUCTION_APP = "production-app"
    PLATFORM_PRODUCTION_APP = "platform.production-app"
    PLATFORM_WALLET_TRANSFERS = "platform.wallet-transfers"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .bankaccountverificationmethod import BankAccountVerificationMethod
from .bankaccountverificationstatus import BankAccountVerificationStatus
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class BankAccountVerificationCreatedTypedDict(TypedDict):
    verification_method: BankAccountVerificationMethod
    status: BankAccountVerificationStatus


class BankAccountVerificationCreated(BaseModel):
    verification_method: Annotated[
        BankAccountVerificationMethod, pydantic.Field(alias="verificationMethod")
    ]

    status: BankAccountVerificationStatus


try:
    BankAccountVerificationCreated.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .paymentlinklineitemimagemetadata import (
    PaymentLinkLineItemImageMetadata,
    PaymentLinkLineItemImageMetadataTypedDict,
)
from .paymentlinklineitemoption import (
    PaymentLinkLineItemOption,
    PaymentLinkLineItemOptionTypedDict,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentLinkLineItemTypedDict(TypedDict):
    r"""Represents a single line item in a payment link, including optional modifiers and quantity."""

    name: str
    r"""The name of the item."""
    base_price: AmountDecimalTypedDict
    r"""The base price of the item before applying option modifiers."""
    quantity: int
    r"""The quantity of this item."""
    options: NotRequired[List[PaymentLinkLineItemOptionTypedDict]]
    r"""Optional list of modifiers applied to this item (e.g., toppings, upgrades, customizations)."""
    images: NotRequired[List[PaymentLinkLineItemImageMetadataTypedDict]]
    r"""Optional list of images associated with this line item."""
    product_id: NotRequired[str]
    r"""Optional unique identifier associating the line item with a product."""


class PaymentLinkLineItem(BaseModel):
    r"""Represents a single line item in a payment link, including optional modifiers and quantity."""

    name: str
    r"""The name of the item."""

    base_price: Annotated[AmountDecimal, pydantic.Field(alias="basePrice")]
    r"""The base price of the item before applying option modifiers."""

    quantity: int
    r"""The quantity of this item."""

    options: Optional[List[PaymentLinkLineItemOption]] = None
    r"""Optional list of modifiers applied to this item (e.g., toppings, upgrades, customizations)."""

    images: Optional[List[PaymentLinkLineItemImageMetadata]] = None
    r"""Optional list of images associated with this line item."""

    product_id: Annotated[Optional[str], pydantic.Field(alias="productID")] = None
    r"""Optional unique identifier associating the line item with a product."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["options", "images", "productID"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PaymentLinkLineItem.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class SECCode(str, Enum):
    r"""Code used to identify the ACH authorization method."""

    WEB = "WEB"
    PPD = "PPD"
    CCD = "CCD"
    TEL = "TEL"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    amountdecimalvalidationerror as components_amountdecimalvalidationerror,
    createinvoicelineitemsvalidationerror as components_createinvoicelineitemsvalidationerror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class CreateInvoiceErrorData(BaseModel):
    customer_account_id: Annotated[
        Optional[str], pydantic.Field(alias="customerAccountID")
    ] = None
    description: Optional[str] = None
    line_items: Annotated[
        Optional[
            components_createinvoicelineitemsvalidationerror.CreateInvoiceLineItemsValidationError
        ],
        pydantic.Field(alias="lineItems"),
    ] = None
    invoice_date: Annotated[Optional[str], pydantic.Field(alias="invoiceDate")] = None
    due_date: Annotated[Optional[str], pydantic.Field(alias="dueDate")] = None
    tax_amount: Annotated[
        Optional[components_amountdecimalvalidationerror.AmountDecimalValidationError],
        pydantic.Field(alias="taxAmount"),
    ] = None


@dataclass(unsafe_hash=True)
class CreateInvoiceError(MoovError):
    data: CreateInvoiceErrorData = field(hash=False)

    def __init__(
        self,
        data: CreateInvoiceErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)
